@@ -207,6 +207,7 @@ void set_resource_limits(void) {
   set_rlimit(RLIMIT_RTPRIO, &opt.rlimit_rtprio);
   set_rlimit(RLIMIT_RTTIME, &opt.rlimit_rttime);
   set_rlimit(RLIMIT_SIGPENDING, &opt.rlimit_sigpending);
+  set_rlimit(RLIMIT_LOCKS, &opt.rlimit_locks);
 }
 
 static const struct app *find_app(const char *name) {
@@ -261,7 +262,8 @@ int main(int argc, char *argv[]) {
 
   if (!(opt.new_ns & CLONE_NEWNS) &&
       (set(OPT_NET_NS) || set(OPT_PRIVATE_RUN) || set(OPT_PRIVATE_TMP) ||
-       set(OPT_RO_SYS) || set(OPT_NEW_ROOT) || set(OPT_PID_NS))) {
+       set(OPT_RO_SYS) || set(OPT_RO_HOME) || set(OPT_RO_ETC) ||
+       set(OPT_NEW_ROOT) || set(OPT_PID_NS))) {
     if (is_verbose())
       fprintf(stderr, "also creating mount namespace implicitly due to other options\n");
     opt.new_ns |= CLONE_NEWNS;
@@ -286,7 +288,14 @@ int main(int argc, char *argv[]) {
     opt.error = true;
 
   if (opt.error) {
-    usage(stderr);
+    const struct option_info *help_option = find_option(OPT_HELP, NULL);
+    if (help_option && opt.app->long_opts && help_option->long_name)
+      fprintf(stderr, "%s: error in options. Run %s --%s for usage\n",
+              program_invocation_short_name,
+              program_invocation_short_name,
+              help_option->long_name);
+    else
+      usage(stderr);
     ret = CHPST_ERROR_OPTIONS;
     goto finish0;
   }
@@ -450,17 +459,21 @@ int main(int argc, char *argv[]) {
     uid_t o = set(OPT_SETUIDGID) ? uid : (uid_t) -1;
     gid_t g = set(OPT_SETUIDGID) ? gid : (gid_t) -1;
 
-    if (set(OPT_RUN_DIR))
-      precreate_dir("/run", 0755, o, g);
+    if (set(OPT_RUN_DIR) &&
+        precreate_dir("/run", 0755, o, g) == -1)
+      goto finish;
 
-    if (set(OPT_STATE_DIR))
-      precreate_dir("/var/lib", 0755, o, g);
+    if (set(OPT_STATE_DIR) &&
+        precreate_dir("/var/lib", 0755, o, g) == -1)
+      goto finish;
 
-    if (set(OPT_CACHE_DIR))
-      precreate_dir("/var/cache", 0755, o, g);
+    if (set(OPT_CACHE_DIR) &&
+        precreate_dir("/var/cache", 0755, o, g) == -1)
+      goto finish;
 
-    if (set(OPT_LOG_DIR))
-      precreate_dir("/var/log", 0755, o, g);
+    if (set(OPT_LOG_DIR) &&
+        precreate_dir("/var/log", 0755, o, g) == -1)
+      goto finish;
   }
 
   if (set(OPT_SETUIDGID) && opt.users_groups.user.resolved) {
@@ -621,23 +634,34 @@ int main(int argc, char *argv[]) {
     special_mount("/proc", "proc", "procfs", NULL);
   }
 
-  if (set(OPT_PRIVATE_RUN))
-    private_mount("/run");
+  if (set(OPT_PRIVATE_RUN) &&
+      private_mount("/run") == -1)
+    goto finish;
 
-  if (set(OPT_PRIVATE_TMP)) {
-    private_mount("/tmp");
-    private_mount("/var/tmp");
-  }
+  if (set(OPT_PRIVATE_TMP) &&
+      (private_mount("/tmp") == -1 ||
+       private_mount("/var/tmp") == -1))
+    goto finish;
 
-  if (set(OPT_PROTECT_HOME)) {
-    private_mount("/home");
-    private_mount("/root");
-    private_mount("/run/user");
-  }
+  if (set(OPT_PROTECT_HOME) &&
+      (private_mount("/home") == -1 ||
+       private_mount("/root") == -1 ||
+       private_mount("/run/user") == -1))
+    goto finish;
 
-  if (set(OPT_RO_SYS)) {
-    remount_sys_ro();
-  }
+  if (!set(OPT_PROTECT_HOME) && set(OPT_RO_HOME) &&
+      (remount_ro("/home") == -1 ||
+       remount_ro("/root") == -1 ||
+       remount_ro("/run/user") == -1))
+    goto finish;
+
+  if (set(OPT_RO_SYS) &&
+      remount_sys_ro() == -1)
+    goto finish;
+
+  if (set(OPT_RO_ETC) &&
+      (remount_ro("/etc") == -1))
+    goto finish;
 
   if (opt.caps_op != CAP_OP_NONE)
     if (!drop_capabilities())
